# -*- encoding: utf-8 -*-
#絵師
require 'spec_helper'

describe Artist do
  before do
    @admin = FactoryGirl.create :admin
    @user = FactoryGirl.create( :user_yas)
    @author = FactoryGirl.create :author, :user_id => @user.id
    @other_user = FactoryGirl.create( :user_yas)
    @other_author = FactoryGirl.create :author, :user_id => @other_user.id
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
  end

  describe '検証に於いて' do
    before do
      @artist = FactoryGirl.build :artist, :author_id => @author.id
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @artist.name = 'a'
        @artist.should be_valid
      end
      it '上限データが通る' do
        @artist.name = 'a'*30
        @artist.should be_valid
      end
    end
    
    context 'nameを検証するとき' do
      it 'nullなら失敗する' do
        @artist.name = nil
        @artist.should_not be_valid
      end
      it '30文字以上なら失敗する' do
        @artist.name = 'a'*31
        @artist.should_not be_valid
      end
    end
    context 'author_idを検証するとき' do
      #絵師は独立絵師があるので数値チェックだけ
      it '数値でなければ失敗する' do
        @artist.author_id = 'a'
        @artist.should_not be_valid
      end
    end
  end
  
  describe 'デフォルト値補充に於いて' do
    it '名前がno nameになっている' do
      @artist = FactoryGirl.build :artist, :name => nil
      @artist.supply_default
      @artist.name.should eq 'no name'
    end
  end
  
  describe '上書き補充に於いて' do
    it '作家idが設定されている' do
      @artist = FactoryGirl.build :artist
      @artist.overwrite @author
      @artist.author_id.should eq @author.id
    end
  end
  
  describe '所持判定に於いて' do
    before do
      @artist = FactoryGirl.create :artist, :author_id => @author.id
      @other_artist = FactoryGirl.create :artist_yas, :author_id => @other_author.id
    end
    it '自分の絵師ならyes' do
      @artist.own?(@author).should == true
    end
    it '他人の絵師ならno' do
      @artist.own?(@other_author).should == false
    end
    it 'パラメータが作家でないならならno' do
      @artist.own?(nil).should == false
    end
  end
  
  describe '閲覧許可に於いて' do
    before do
      @artist = FactoryGirl.create :artist, :author_id => @author.id
    end
    context '検査対象がnil(ゲスト)のとき' do
      context 'クローズドモードのとき' do
        before do
          MagicNumber['run_mode'] = 1
        end
        it '不許可を返す。' do
          r = @artist.visible?(nil)
          r.should be_false
        end
      end
      context 'オープンモードのとき' do
        before do
          MagicNumber['run_mode'] = 0
        end
        it '許可する' do
          r = @artist.visible?(nil)
          r.should be_true
        end
      end
    end
    context '検査対象が作家のとき' do
      it '許可する' do
        r = @artist.visible?(@author)
        r.should == true
      end
    end
    context '検査対象がそれ以外のとき' do
      it '不許可を返す。' do
        r = @artist.visible?(@admin)
        r.should be_false
      end
    end
  end
  
  describe '一覧取得に於いて' do
    before do
      @artist = FactoryGirl.create :artist, :author_id => @author.id
    end
    context 'page補正について' do
      it '文字列から数値に変換される' do
        Artist.page('8').should eq 8
      end
      it 'nilの場合は1になる' do
        Artist.page().should eq 1
      end
      it '0以下の場合は1になる' do
        Artist.page('0').should eq 1
      end
    end
    context 'page_size補正について' do
      it '文字列から数値に変換される' do
        Artist.page_size('7').should eq 7
      end
      it 'nilの場合はArtist.default_page_sizeになる' do
        Artist.page_size().should eq Artist.default_page_size
      end
      it '0以下の場合はArtist.default_page_sizeになる' do
        Artist.page_size('0').should eq Artist.default_page_size
      end
      it 'Artist.max_page_sizeを超えた場合はArtist.max_page_sizeになる' do
        Artist.page_size('1000').should eq Artist.max_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it '一覧取得オプションを利用している' do
        Artist.stub(:list_opt).with(any_args).and_return({})
        Artist.should_receive(:list_opt).with(any_args).exactly(1)
        r = Artist.list
      end
    end
    it 'リストを返す' do
      r = Artist.list
      r.should eq [@artist]
    end
    it '作成時系列で並んでいる' do
      n = FactoryGirl.create :artist, :author_id => @author.id, :name => 'artist', :created_at => Time.now + 100
      l = Artist.list
      l.should eq [n, @artist]
    end
    context 'DBに5件あって1ページの件数を2件に変えたとして' do
      before do
        @artist2 = FactoryGirl.create :artist, :author_id => @author.id, :name => 'artist2', :created_at => Time.now + 100
        @artist3 = FactoryGirl.create :artist, :author_id => @author.id, :name => 'artist3', :created_at => Time.now + 200
        @artist4 = FactoryGirl.create :artist, :author_id => @author.id, :name => 'artist4', :created_at => Time.now + 300
        @artist5 = FactoryGirl.create :artist, :author_id => @author.id, :name => 'artist5', :created_at => Time.now + 400
        Artist.stub(:default_page_size).and_return(2)
      end
      it '通常は2件を返す' do
        r = Artist.list
        r.should have(2).items 
      end
      it 'page=1なら末尾2件を返す' do
        #時系列で並んでいる
        r = Artist.list(1)
        r.should eq [@artist5, @artist4]
      end
      it 'page=2なら中間2件を返す' do
        r = Artist.list(2)
        r.should eq [@artist3, @artist2]
      end
      it 'page=3なら先頭1件を返す' do
        r = Artist.list(3)
        r.should eq [@artist]
      end
    end
    context 'DBに5件あって1ページの件数を0件に変えたとして' do
      before do
        @artist2 = FactoryGirl.create :artist, :author_id => @author.id, :name => 'artist2', :created_at => Time.now + 100
        @artist3 = FactoryGirl.create :artist, :author_id => @author.id, :name => 'artist3', :created_at => Time.now + 200
        @artist4 = FactoryGirl.create :artist, :author_id => @author.id, :name => 'artist4', :created_at => Time.now + 300
        @artist5 = FactoryGirl.create :artist, :author_id => @author.id, :name => 'artist5', :created_at => Time.now + 400
        Artist.stub(:default_page_size).and_return(2)
      end
      it '通常は全件(5件)を返す' do
        r = Artist.list 5, 0
        r.should have(5).items 
      end
    end
  end
  describe '一覧取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Artist.list_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = Artist.list_opt[:include]
      r.should have(1).items
    end
    it '作家を含んでいる' do
      r = Artist.list_opt[:include]
      r.has_key?(:author).should be_true
    end
  end
  describe 'json一覧出力オプションに於いて' do
    before do
      @artist = FactoryGirl.create :artist, :author_id => @author.id
    end
    it '作家を含んでいる' do
      r = Artist.list.to_json Artist.list_json_opt
      j = JSON.parse r
      i = j.first
      i.has_key?('author').should be_true
    end
  end
  
  describe '単体取得に於いて' do
    before do
      @artist = FactoryGirl.create :artist, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        Artist.stub(:show_opt).with(any_args).and_return({})
        Artist.should_receive(:show_opt).with(any_args).exactly(1)
        r = Artist.show @artist.id, @author
      end
      it '閲覧許可を問い合わせている' do
        Artist.any_instance.stub(:visible?).with(any_args).and_return(true)
        Artist.any_instance.should_receive(:visible?).with(any_args).exactly(1)
        r = Artist.show @artist.id, @author
      end
    end
    it '指定の絵師を返す' do
      a = Artist.show @artist.id, @author
      a.should eq @artist
    end
    context '閲覧許可が出なかったとき' do
      it '403Forbidden例外を返す' do
        Artist.any_instance.stub(:visible?).and_return(false)
        lambda{
          Artist.show @artist.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しない絵師を開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Artist.show 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '編集取得に於いて' do
    before do
      @artist = FactoryGirl.create :artist, :author_id => @author.id
    end
    context 'つつがなく終わるとき' do
      it '単体取得オプションを利用している' do
        Artist.stub(:show_opt).with(any_args).and_return({})
        Artist.should_receive(:show_opt).with(any_args).exactly(1)
        r = Artist.edit @artist.id, @author
      end
      it '所持判定を問い合わせている' do
        Artist.any_instance.stub(:own?).with(any_args).and_return(true)
        Artist.any_instance.should_receive(:own?).with(any_args).exactly(1)
        r = Artist.edit @artist.id, @author
      end
    end
    it '指定の絵師を返す' do
      Artist.any_instance.stub(:own?).and_return(true)
      r = Artist.edit @artist.id, @author.id
      r.should eq @artist
    end
    context '他人の絵師を開こうとしたとき' do
      it '403Forbidden例外を返す' do
        Artist.any_instance.stub(:own?).and_return(false)
        lambda{
          Artist.edit @artist.id, @author
        }.should raise_error(ActiveRecord::Forbidden)
      end
    end
    context '存在しない絵師を開こうとしたとき' do
      it '404RecordNotFound例外を返す' do
        lambda{
          Artist.edit 110, @author
        }.should raise_error(ActiveRecord::RecordNotFound)
      end
    end
  end
  describe '単体取得オプションに於いて' do
    it 'includeキーを含んでいる' do
      r = Artist.show_opt
      r.has_key?(:include).should be_true
    end
    it '1つの項目を含んでいる' do
      r = Artist.show_opt[:include]
      r.should have(1).items
    end
  end
  describe 'json単体出力オプションに於いて' do
    before do
      @artist = FactoryGirl.create :artist, :author_id => @author.id
    end
    it '作家を含んでいる' do
      r = Artist.show(@artist.id, @author).to_json Artist.show_json_opt
      j = JSON.parse r
      i = j
      i.has_key?('author').should be_true
    end
  end
end
